#region Copyright
/*
 * Copyright 2005-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
#endregion

using System.Data;
using Seasar.Extension.ADO;
using Seasar.Extension.ADO.Impl;
using Seasar.Extension.DataSets;
using Seasar.Extension.DataSets.Impl;
using Seasar.Framework.Exceptions;
using Seasar.Framework.Unit;
using Seasar.Framework.Util;
using IDataReader = Seasar.Extension.DataSets.IDataReader;

namespace Seasar.Extension.Unit
{
    public class S2TestCase : S2FrameworkTestCaseBase
    {
        private IDataSource _dataSource;
        private IDbConnection _connection;
        private ICommandFactory _commandFactory;

        public IDataSource DataSource
        {
            get
            {
                if (_dataSource == null)
                {
                    throw new EmptyRuntimeException("_dataSource");
                }
                return _dataSource;
            }
        }

        public IDbConnection Connection
        {
            get
            {
                if (_connection != null)
                {
                    return _connection;
                }
                _connection = DataSourceUtil.GetConnection(_dataSource);
                return _connection;
            }
        }

        public bool HasConnection
        {
            get { return _connection != null; }
        }

        public ICommandFactory CommandFactory
        {
            get
            {
                if (_commandFactory == null)
                {
                    _commandFactory = Container.GetComponent(typeof(ICommandFactory)) as ICommandFactory;
                }
                if (_commandFactory == null)
                {
                    _commandFactory = BasicCommandFactory.INSTANCE;
                }
                return _commandFactory;
            }
        }

        internal void SetConnection(IDbConnection connection)
        {
            _connection = connection;
        }

        internal void SetDataSource(IDataSource dataSource)
        {
            _dataSource = dataSource;
        }

        /// <summary>
        /// Excelt@Cǂ݁ADataSet쐬܂B
        /// V[ge[uAsڂJAsڈȍ~f[^ Ƃēǂݍ݂܂B
        /// 
        /// pXAssemblyŎw肳ĂfBNg[gƂB
        /// ݒt@C̐΃pXAt@Ĉ݂w肵܂B
        /// t@Ĉ݂̏ꍇAeXgP[XƓpbP[Wɂ̂Ƃ܂B
        /// <seealso cref="Seasar.Extension.DataSets.Impl.XlsReader.Read"/>
        /// </summary>
        /// <param name="path">Excelt@C̃pX</param>
        /// <returns>Excelt@C̓e쐬DataSet</returns>
        public virtual DataSet ReadXls(string path)
        {
            IDataReader reader = new XlsReader(ConvertPath(path));
            return reader.Read();
        }

        /// <summary>
        /// DataSet̓eAExcelt@C쐬܂B
        /// V[gɃe[uAsڂɃJAsڈȍ~Ƀf[^ ݂܂B
        /// 
        /// pXAssemblyŎw肳ĂfBNg[gƂB
        /// ݒt@C̐΃pXAt@Ĉ݂w肵܂B
        /// t@Ĉ݂̏ꍇAeXgP[XƓpbP[Wɂ̂Ƃ܂B
        /// <seealso cref="Seasar.Extension.DataSets.Impl.XlsWriter.Write"/>
        /// </summary>
        /// <param name="path">Excelt@C̃pX</param>
        /// <param name="dataSet">Excelt@CɏޓeDataSet</param>
        public virtual void WriteXls(string path, DataSet dataSet)
        {
            IDataWriter writer = new XlsWriter(ConvertPath(path));
            writer.Write(dataSet);
        }

        /// <summary>
        /// DataSetDBɏ݂܂B
        /// <seealso cref="Seasar.Extension.DataSets.Impl.SqlWriter.Write"/>
        /// </summary>
        /// <param name="dataSet">f[^x[XɏޓeDataSet</param>
        public virtual void WriteDb(DataSet dataSet)
        {
            IDataWriter writer = SqlWriterFactory.GetSqlWriter(DataSource, CommandFactory);
            writer.Write(dataSet);
        }

        /// <summary>
        /// DB烌R[hǂݍ݁ADataTable쐬܂B
        /// <seealso cref="Seasar.Extension.DataSets.Impl.SqlTableReader.Read"/>
        /// </summary>
        /// <param name="table">ǂݍރe[u</param>
        /// <returns>ǂݍ񂾓e쐬DataTable</returns>
        public virtual DataTable ReadDbByTable(string table)
        {
            return ReadDbByTable(table, null);
        }

        /// <summary>
        /// DB烌R[hǂݍ݁ADataTable쐬܂B
        /// ǂݍރR[hcondition̏𖞂R[hłB conditionɂ" WHERE "ZbgĂB
        /// <seealso cref="Seasar.Extension.DataSets.Impl.SqlTableReader.Read"/>
        /// </summary>
        /// <param name="table">ǂݍރe[u</param>
        /// <param name="condition">(WHERĚ)</param>
        /// <returns>ǂݍ񂾓e쐬DataTable</returns>
        public virtual DataTable ReadDbByTable(string table, string condition)
        {
            SqlTableReader reader = new SqlTableReader(DataSource);
            reader.SetTable(table, condition);
            return reader.Read();
        }

        /// <summary>
        /// DBSQL̎sʂ擾ADataTable쐬܂B
        /// 쐬DataTablẽe[utableNameɂȂ܂B
        /// <seealso cref="Seasar.Extension.DataSets.Impl.SqlTableReader.Read"/>
        /// </summary>
        /// <param name="sql">sSQL</param>
        /// <param name="tableName">쐬DataTablẽe[u</param>
        /// <returns>ǂݏoeDataTable</returns>
        public virtual DataTable ReadDbBySql(string sql, string tableName)
        {
            SqlTableReader reader = new SqlTableReader(DataSource);
            reader.SetSql(sql, tableName);
            return reader.Read();

        }

        /// <summary>
        /// Excelt@Cǂݍ݁ADBɏ݂܂B
        /// V[ge[uAsڂJAsڈȍ~f[^ Ƃēǂݍ݂܂B
        /// 
        /// pXAssemblyŎw肳ĂfBNg[gƂB
        /// ݒt@C̐΃pXAt@Ĉ݂w肵܂B
        /// t@Ĉ݂̏ꍇAeXgP[XƓpbP[Wɂ̂Ƃ܂B
        /// <seealso cref="Seasar.Extension.DataSets.Impl.XlsReader.Read"/>
        /// <seealso cref="Seasar.Extension.DataSets.Impl.SqlWriter.Write"/>
        /// </summary>
        /// <param name="path">Excelt@C̃pX</param>
        public virtual void ReadXlsWriteDb(string path)
        {
            WriteDb(ReadXls(path));
        }

        /// <summary>
        /// Excelt@Cǂݍ݁ADBɏ݂܂B
        /// V[ge[uAsڂJAsڈȍ~f[^ Ƃēǂݍ݂܂B
        /// Excel̓eDB̃R[hƂŎL[v̂΁A ̃R[h폜ɏ݂܂B
        /// 
        /// pXAssemblyŎw肳ĂfBNg[gƂB
        /// ݒt@C̐΃pXAt@Ĉ݂w肵܂B
        /// t@Ĉ݂̏ꍇAeXgP[XƓpbP[Wɂ̂Ƃ܂B
        /// <seealso cref="Seasar.Extension.DataSets.Impl.XlsReader.Read"/>
        /// <seealso cref="Seasar.Extension.DataSets.Impl.SqlWriter.Write"/>
        /// </summary>
        /// <param name="path">Excelt@C̃pX</param>
        public virtual void ReadXlsReplaceDb(string path)
        {
            DataSet dataSet = ReadXls(path);
            DeleteDb(dataSet);
            WriteDb(dataSet);
        }

        /// <summary>
        /// Excelt@Cǂݍ݁ADBɏ݂܂B
        /// V[ge[uAsڂJAsڈȍ~f[^ Ƃēǂݍ݂܂B
        /// ΏۂƂȂe[ũR[hSč폜ɏ݂܂B
        /// 
        /// pXAssemblyŎw肳ĂfBNg[gƂB
        /// ݒt@C̐΃pXAt@Ĉ݂w肵܂B
        /// t@Ĉ݂̏ꍇAeXgP[XƓpbP[Wɂ̂Ƃ܂B
        /// <seealso cref="Seasar.Extension.DataSets.Impl.XlsReader.Read"/>
        /// <seealso cref="Seasar.Extension.DataSets.Impl.SqlWriter.Write"/>
        /// </summary>
        /// <param name="path">Excelt@C̃pX</param>
        public virtual void ReadXlsAllReplaceDb(string path)
        {
            DataSet dataSet = ReadXls(path);
            for (int i = dataSet.Tables.Count - 1; i >= 0; --i)
            {
                DeleteTable(dataSet.Tables[i].TableName);
            }
            WriteDb(dataSet);
        }

        /// <summary>
        /// DataSetɑΉDB̃R[hǂݍ݁ADataSet쐬܂ B
        /// <seealso cref="Seasar.Extension.DataSets.Impl.SqlReloadReader.Read"/>
        /// </summary>
        /// <param name="dataSet">ΏDBɑΉDataSet</param>
        /// <returns>ŐVԂDataSet</returns>
        public virtual DataSet Reload(DataSet dataSet)
        {
            return new SqlReloadReader(DataSource, dataSet).Read();
        }

        /// <summary>
        /// DataTableɑΉDB̃R[hǂݍ݁ADataTable쐬 ܂B
        /// <seealso cref="Seasar.Extension.DataSets.Impl.SqlReloadReader.Read"/>
        /// </summary>
        /// <param name="table">ΏDBɑΉDataTable</param>
        /// <returns>ŐVԂDataTable</returns>
        public virtual DataTable Reload(DataTable table)
        {
            return new SqlReloadTableReader(DataSource, table).Read();
        }

        /// <summary>
        /// DataSetɑΉDB̃R[h폜܂B
        /// <seealso cref="Seasar.Extension.DataSets.Impl.SqlDeleteTableWriter.Write"/>
        /// </summary>
        /// <param name="dataSet">ΏDBɑΉDataSet</param>
        public virtual void DeleteDb(DataSet dataSet)
        {
            SqlDeleteTableWriter writer = new SqlDeleteTableWriter(DataSource);
            for (int i = dataSet.Tables.Count - 1; i >= 0; --i)
            {
                writer.Write(dataSet.Tables[i]);
            }
        }

        /// <summary>
        /// DBw肷e[ȗSR[h폜܂B
        /// </summary>
        /// <param name="tableName">폜Ώۂ̃e[u</param>
        public virtual void DeleteTable(string tableName)
        {
            IUpdateHandler handler = new BasicUpdateHandler(
                DataSource,
                "DELETE FROM " + tableName
                );
            handler.Execute(null);
        }
    }
}
